import * as Fs from "node:fs/promises";
const byteAlignment = 8;
const defaultBufferSize = 0x40000;
const maxBlockSize = 0x4000;
function assert(cond) {
    if (!cond) {
        throw new Error("Assertion failed.");
    }
}
export class Reader {
    static async open(fileName, options) {
        var _a;
        const fileHandle = await Fs.open(fileName);
        const reader = new Reader();
        reader.fileHandle = fileHandle;
        reader.isOpen = true;
        const bufferSizeArg = (options === null || options === void 0 ? void 0 : options.bufferSize) || defaultBufferSize;
        reader.buffer = (_a = options === null || options === void 0 ? void 0 : options.preallocatedBuffer) !== null && _a !== void 0 ? _a : Buffer.allocUnsafe(bufferSizeArg);
        assert(reader.buffer.length >= 2 * byteAlignment);
        reader.bufferEod = 0;
        reader.bufferFilePos = 0;
        reader.currentFilePos = 0;
        reader.eof = false;
        reader.blockSize = Math.min(maxBlockSize, roundDownToPowerOf2(Math.floor(reader.buffer.length / 4)));
        return reader;
    }
    async close() {
        if (!this.isOpen) {
            return;
        }
        this.isOpen = false;
        await this.fileHandle.close();
    }
    async get(length) {
        assert(this.isOpen);
        assert(length >= 0);
        assert(this.currentFilePos >= this.bufferFilePos);
        if (length == 0) {
            return Buffer.alloc(0);
        }
        if (this.currentFilePos + length > this.bufferFilePos + this.bufferEod) {
            await this.loadDataIntoBuffer();
        }
        const currentOffs = this.currentFilePos - this.bufferFilePos;
        assert(currentOffs >= 0);
        const availableData = this.bufferEod - currentOffs;
        if (availableData <= 0) {
            assert(this.eof);
            return undefined;
        }
        return Buffer.from(this.buffer.buffer, this.buffer.byteOffset + currentOffs, availableData);
    }
    advance(length) {
        assert(length >= 0);
        this.currentFilePos += length;
    }
    async loadDataIntoBuffer() {
        if (this.eof) {
            return;
        }
        this.moveBufferDataDown();
        while (true) {
            const freeLen = this.buffer.length - this.bufferEod;
            assert(freeLen >= 0);
            if (freeLen == 0) {
                return;
            }
            const readFilePos = this.bufferFilePos + this.bufferEod;
            const blockEndPos = Math.floor((readFilePos + freeLen) / this.blockSize) * this.blockSize;
            const blockReadLen = blockEndPos - readFilePos;
            const readLen = blockReadLen > 0 ? blockReadLen : freeLen;
            const r = await this.fileHandle.read(this.buffer, this.bufferEod, readLen, readFilePos);
            assert(r.bytesRead <= readLen);
            if (r.bytesRead <= 0) {
                this.eof = true;
                return;
            }
            this.bufferEod += r.bytesRead;
            if (r.bytesRead == readLen) {
                return;
            }
        }
    }
    moveBufferDataDown() {
        assert(this.currentFilePos >= this.bufferFilePos);
        const keepOffs = this.currentFilePos - this.bufferFilePos;
        const keepLen = this.bufferEod - keepOffs;
        const alignmentOffs = this.currentFilePos % byteAlignment;
        if (keepLen <= 0) {
            this.bufferFilePos = this.currentFilePos - alignmentOffs;
            this.bufferEod = alignmentOffs;
            return;
        }
        if (alignmentOffs >= keepOffs) {
            return;
        }
        this.buffer.copy(this.buffer, alignmentOffs, keepOffs, keepOffs + keepLen);
        this.bufferFilePos = this.currentFilePos - alignmentOffs;
        this.bufferEod = alignmentOffs + keepLen;
    }
    get filePosition() {
        return this.currentFilePos;
    }
}
function roundDownToPowerOf2(x) {
    let n = 1;
    while (2 * n <= x) {
        n *= 2;
    }
    return n;
}
